% Copyright (C) 2003 Csaba Szepesvari
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2.1 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

% -*- texinfo -*-
% @deftypefn {Function File} {@var{funname} = } funspec (@var{args}, @var{retvals}, @var{body})
% @code{funspec} allows you to define temporary function files.
% It returns the name of the created temporary function file that can be
% called using octave's builtin @code{feval} facility.
%
% @var{args} specifies the argument names (it is a string of comma separated list of argument names).
% @var{retvals} specifies the return values (it is a string of comma separated list of return variable names).
% @var{body} specifies the body of the function (it is a string of a list of ';' separated octave commmands).
%
% By default @code{funspec} creates the function files in the 'funspec/tmp' directory of the user's
% home directory. Ideally, this directory should be put on @var{LOADPATH}. The user can control where
% to place these files by specifying a value in @var{FUNSPECPATH}.
%
% Example - creating and evaluating an identity function:
% @example
%   tmpfun = funspec ('x', 'y', 'y=x');
%   feval( tmpfun, 3 )
%   feval( tmpfun, [1 2 3] )
% @end example
%
% @end deftypefn
% @seealso{feval}

% Author: Csaba Szepesvari (www.sztaki.hu/~szcsaba)
% Description: Creating temporary function specification files.
% Keywords: lambda calculus

function funname = funspec(args,retvals,body)

if (exist('FUNSPECPATH')==0)
  FUNSPECPATH = 'funspec/tmp/' ;
end;
tmp = tempname;
[pathstr,funname,ext,versn]=fileparts(tmp);
mfunname = strcat(funname,'.m');
mfunname = strcat(FUNSPECPATH,mfunname);
fprintf('%s\n', mfunname);
fid = fopen (mfunname, 'w','native');
if (fid<0)
  fprintf('funspec: tried to create temporary %s',mfunname);
  error('funspec: error creating temporary!');
end;
fprintf(fid,'function [%s] = %s(%s)\n',retvals,funname,args);
fprintf(fid,'%s;\n',body);
%fprintf(fid,'end\n');
fclose(fid);
